# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["UserAddressListParams", "Filter", "FilterCustomerReference", "FilterStreetAddress"]


class UserAddressListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[customer_reference][eq],
    filter[customer_reference][contains], filter[street_address][contains]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]

    sort: Literal["created_at", "first_name", "last_name", "business_name", "street_address"]
    """Specifies the sort order for results.

    By default sorting direction is ascending. To have the results sorted in
    descending order add the <code> -</code> prefix.<br/><br/> That is: <ul>

      <li>
        <code>street_address</code>: sorts the result by the
        <code>street_address</code> field in ascending order.
      </li>

      <li>
        <code>-street_address</code>: sorts the result by the
        <code>street_address</code> field in descending order.
      </li>
    </ul> <br/> If not given, results are sorted by <code>created_at</code> in descending order.
    """


class FilterCustomerReference(TypedDict, total=False):
    """Filter user addresses via the customer reference.

    Supports both exact matching (eq) and partial matching (contains). Matching is not case-sensitive.
    """

    contains: str
    """
    If present, user addresses with <code>customer_reference</code> containing the
    given value will be returned. Matching is not case-sensitive.
    """

    eq: str
    """Filter user addresses via exact customer reference match.

    Matching is not case-sensitive.
    """


class FilterStreetAddress(TypedDict, total=False):
    """Filter user addresses via street address.

    Supports partial matching (contains). Matching is not case-sensitive.
    """

    contains: str
    """
    If present, user addresses with <code>street_address</code> containing the given
    value will be returned. Matching is not case-sensitive. Requires at least three
    characters.
    """


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[customer_reference][eq], filter[customer_reference][contains], filter[street_address][contains]
    """

    customer_reference: FilterCustomerReference
    """Filter user addresses via the customer reference.

    Supports both exact matching (eq) and partial matching (contains). Matching is
    not case-sensitive.
    """

    street_address: FilterStreetAddress
    """Filter user addresses via street address.

    Supports partial matching (contains). Matching is not case-sensitive.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["ShortCodeListParams", "Filter"]


class ShortCodeListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[messaging_profile_id]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[messaging_profile_id]
    """

    messaging_profile_id: str
    """Filter by Messaging Profile ID.

    Use the string `null` for phone numbers without assigned profiles. A synonym for
    the `/messaging_profiles/{id}/short_codes` endpoint when querying about an
    extant profile.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .voice import (
    VoiceResource,
    AsyncVoiceResource,
    VoiceResourceWithRawResponse,
    AsyncVoiceResourceWithRawResponse,
    VoiceResourceWithStreamingResponse,
    AsyncVoiceResourceWithStreamingResponse,
)
from .messaging import (
    MessagingResource,
    AsyncMessagingResource,
    MessagingResourceWithRawResponse,
    AsyncMessagingResourceWithRawResponse,
    MessagingResourceWithStreamingResponse,
    AsyncMessagingResourceWithStreamingResponse,
)
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from .speech_to_text import (
    SpeechToTextResource,
    AsyncSpeechToTextResource,
    SpeechToTextResourceWithRawResponse,
    AsyncSpeechToTextResourceWithRawResponse,
    SpeechToTextResourceWithStreamingResponse,
    AsyncSpeechToTextResourceWithStreamingResponse,
)

__all__ = ["BatchDetailRecordsResource", "AsyncBatchDetailRecordsResource"]


class BatchDetailRecordsResource(SyncAPIResource):
    @cached_property
    def messaging(self) -> MessagingResource:
        """Messaging batch detail records"""
        return MessagingResource(self._client)

    @cached_property
    def speech_to_text(self) -> SpeechToTextResource:
        """Speech to text batch detail records"""
        return SpeechToTextResource(self._client)

    @cached_property
    def voice(self) -> VoiceResource:
        """Voice batch detail records"""
        return VoiceResource(self._client)

    @cached_property
    def with_raw_response(self) -> BatchDetailRecordsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return BatchDetailRecordsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BatchDetailRecordsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return BatchDetailRecordsResourceWithStreamingResponse(self)


class AsyncBatchDetailRecordsResource(AsyncAPIResource):
    @cached_property
    def messaging(self) -> AsyncMessagingResource:
        """Messaging batch detail records"""
        return AsyncMessagingResource(self._client)

    @cached_property
    def speech_to_text(self) -> AsyncSpeechToTextResource:
        """Speech to text batch detail records"""
        return AsyncSpeechToTextResource(self._client)

    @cached_property
    def voice(self) -> AsyncVoiceResource:
        """Voice batch detail records"""
        return AsyncVoiceResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBatchDetailRecordsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncBatchDetailRecordsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBatchDetailRecordsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncBatchDetailRecordsResourceWithStreamingResponse(self)


class BatchDetailRecordsResourceWithRawResponse:
    def __init__(self, batch_detail_records: BatchDetailRecordsResource) -> None:
        self._batch_detail_records = batch_detail_records

    @cached_property
    def messaging(self) -> MessagingResourceWithRawResponse:
        """Messaging batch detail records"""
        return MessagingResourceWithRawResponse(self._batch_detail_records.messaging)

    @cached_property
    def speech_to_text(self) -> SpeechToTextResourceWithRawResponse:
        """Speech to text batch detail records"""
        return SpeechToTextResourceWithRawResponse(self._batch_detail_records.speech_to_text)

    @cached_property
    def voice(self) -> VoiceResourceWithRawResponse:
        """Voice batch detail records"""
        return VoiceResourceWithRawResponse(self._batch_detail_records.voice)


class AsyncBatchDetailRecordsResourceWithRawResponse:
    def __init__(self, batch_detail_records: AsyncBatchDetailRecordsResource) -> None:
        self._batch_detail_records = batch_detail_records

    @cached_property
    def messaging(self) -> AsyncMessagingResourceWithRawResponse:
        """Messaging batch detail records"""
        return AsyncMessagingResourceWithRawResponse(self._batch_detail_records.messaging)

    @cached_property
    def speech_to_text(self) -> AsyncSpeechToTextResourceWithRawResponse:
        """Speech to text batch detail records"""
        return AsyncSpeechToTextResourceWithRawResponse(self._batch_detail_records.speech_to_text)

    @cached_property
    def voice(self) -> AsyncVoiceResourceWithRawResponse:
        """Voice batch detail records"""
        return AsyncVoiceResourceWithRawResponse(self._batch_detail_records.voice)


class BatchDetailRecordsResourceWithStreamingResponse:
    def __init__(self, batch_detail_records: BatchDetailRecordsResource) -> None:
        self._batch_detail_records = batch_detail_records

    @cached_property
    def messaging(self) -> MessagingResourceWithStreamingResponse:
        """Messaging batch detail records"""
        return MessagingResourceWithStreamingResponse(self._batch_detail_records.messaging)

    @cached_property
    def speech_to_text(self) -> SpeechToTextResourceWithStreamingResponse:
        """Speech to text batch detail records"""
        return SpeechToTextResourceWithStreamingResponse(self._batch_detail_records.speech_to_text)

    @cached_property
    def voice(self) -> VoiceResourceWithStreamingResponse:
        """Voice batch detail records"""
        return VoiceResourceWithStreamingResponse(self._batch_detail_records.voice)


class AsyncBatchDetailRecordsResourceWithStreamingResponse:
    def __init__(self, batch_detail_records: AsyncBatchDetailRecordsResource) -> None:
        self._batch_detail_records = batch_detail_records

    @cached_property
    def messaging(self) -> AsyncMessagingResourceWithStreamingResponse:
        """Messaging batch detail records"""
        return AsyncMessagingResourceWithStreamingResponse(self._batch_detail_records.messaging)

    @cached_property
    def speech_to_text(self) -> AsyncSpeechToTextResourceWithStreamingResponse:
        """Speech to text batch detail records"""
        return AsyncSpeechToTextResourceWithStreamingResponse(self._batch_detail_records.speech_to_text)

    @cached_property
    def voice(self) -> AsyncVoiceResourceWithStreamingResponse:
        """Voice batch detail records"""
        return AsyncVoiceResourceWithStreamingResponse(self._batch_detail_records.voice)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .outbound_message import OutboundMessage

__all__ = ["DeliveryUpdateWebhookEvent", "Meta"]


class Meta(BaseModel):
    attempt: Optional[int] = None
    """Number of attempts to deliver the webhook event."""

    delivered_to: Optional[str] = None
    """The webhook URL the event was delivered to."""


class DeliveryUpdateWebhookEvent(BaseModel):
    data: Optional[OutboundMessage] = None

    meta: Optional[Meta] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .shared.whatsapp_template_data import WhatsappTemplateData

__all__ = ["WhatsappMessageTemplateUpdateResponse"]


class WhatsappMessageTemplateUpdateResponse(BaseModel):
    data: Optional[WhatsappTemplateData] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from datetime import datetime

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncDefaultPaginationForMessagingTollfree, AsyncDefaultPaginationForMessagingTollfree
from ...._base_client import AsyncPaginator, make_request_options
from ....types.messaging_tollfree.verification import (
    Volume,
    UseCaseCategories,
    TfVerificationStatus,
    TollFreeVerificationEntityType,
    request_list_params,
    request_create_params,
    request_update_params,
    request_retrieve_status_history_params,
)
from ....types.messaging_tollfree.verification.volume import Volume
from ....types.messaging_tollfree.verification.url_param import URLParam
from ....types.messaging_tollfree.verification.use_case_categories import UseCaseCategories
from ....types.messaging_tollfree.verification.tf_phone_number_param import TfPhoneNumberParam
from ....types.messaging_tollfree.verification.tf_verification_status import TfVerificationStatus
from ....types.messaging_tollfree.verification.verification_request_egress import VerificationRequestEgress
from ....types.messaging_tollfree.verification.verification_request_status import VerificationRequestStatus
from ....types.messaging_tollfree.verification.toll_free_verification_entity_type import TollFreeVerificationEntityType
from ....types.messaging_tollfree.verification.request_retrieve_status_history_response import (
    RequestRetrieveStatusHistoryResponse,
)

__all__ = ["RequestsResource", "AsyncRequestsResource"]


class RequestsResource(SyncAPIResource):
    """Manage your tollfree verification requests"""

    @cached_property
    def with_raw_response(self) -> RequestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return RequestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> RequestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return RequestsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        additional_information: str,
        business_addr1: str,
        business_city: str,
        business_contact_email: str,
        business_contact_first_name: str,
        business_contact_last_name: str,
        business_contact_phone: str,
        business_name: str,
        business_state: str,
        business_zip: str,
        corporate_website: str,
        message_volume: Volume,
        opt_in_workflow: str,
        opt_in_workflow_image_urls: Iterable[URLParam],
        phone_numbers: Iterable[TfPhoneNumberParam],
        production_message_content: str,
        use_case: UseCaseCategories,
        use_case_summary: str,
        age_gated_content: bool | Omit = omit,
        business_addr2: str | Omit = omit,
        business_registration_country: Optional[str] | Omit = omit,
        business_registration_number: Optional[str] | Omit = omit,
        business_registration_type: Optional[str] | Omit = omit,
        campaign_verify_authorization_token: Optional[str] | Omit = omit,
        doing_business_as: Optional[str] | Omit = omit,
        entity_type: Optional[TollFreeVerificationEntityType] | Omit = omit,
        help_message_response: Optional[str] | Omit = omit,
        isv_reseller: Optional[str] | Omit = omit,
        opt_in_confirmation_response: Optional[str] | Omit = omit,
        opt_in_keywords: Optional[str] | Omit = omit,
        privacy_policy_url: Optional[str] | Omit = omit,
        terms_and_condition_url: Optional[str] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRequestEgress:
        """
        Submit a new tollfree verification request

        Args:
          additional_information: Any additional information

          business_addr1: Line 1 of the business address

          business_city: The city of the business address; the first letter should be capitalized

          business_contact_email: The email address of the business contact

          business_contact_first_name: First name of the business contact; there are no specific requirements on
              formatting

          business_contact_last_name: Last name of the business contact; there are no specific requirements on
              formatting

          business_contact_phone: The phone number of the business contact in E.164 format

          business_name: Name of the business; there are no specific formatting requirements

          business_state: The full name of the state (not the 2 letter code) of the business address; the
              first letter should be capitalized

          business_zip: The ZIP code of the business address

          corporate_website: A URL, including the scheme, pointing to the corporate website

          message_volume: Message Volume Enums

          opt_in_workflow: Human-readable description of how end users will opt into receiving messages
              from the given phone numbers

          opt_in_workflow_image_urls: Images showing the opt-in workflow

          phone_numbers: The phone numbers to request the verification of

          production_message_content: An example of a message that will be sent from the given phone numbers

          use_case: Tollfree usecase categories

          use_case_summary: Human-readable summary of the desired use-case

          age_gated_content: Indicates if messaging content requires age gating (e.g., 18+). Defaults to
              false if not provided.

          business_addr2: Line 2 of the business address

          business_registration_country: ISO 3166-1 alpha-2 country code of the issuing business authority. Must be
              exactly 2 letters. Automatically converted to uppercase. Required from
              January 2026.

          business_registration_number: Official business registration number (e.g., Employer Identification Number
              (EIN) in the U.S.). Required from January 2026.

          business_registration_type: Type of business registration being provided. Required from January 2026.

          campaign_verify_authorization_token: Campaign Verify Authorization Token required for Political use case submissions
              starting February 17, 2026. This token is validated by Zipwhip and must be
              provided for all Political use case verifications after the deadline.

          doing_business_as: Doing Business As (DBA) name if different from legal name

          entity_type: Business entity classification

          help_message_response: The message returned when users text 'HELP'

          isv_reseller: ISV name

          opt_in_confirmation_response: Message sent to users confirming their opt-in to receive messages

          opt_in_keywords: Keywords used to collect and process consumer opt-ins

          privacy_policy_url: URL pointing to the business's privacy policy. Plain string, no URL format
              validation.

          terms_and_condition_url: URL pointing to the business's terms and conditions. Plain string, no URL format
              validation.

          webhook_url: URL that should receive webhooks relating to this verification request

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/messaging_tollfree/verification/requests",
            body=maybe_transform(
                {
                    "additional_information": additional_information,
                    "business_addr1": business_addr1,
                    "business_city": business_city,
                    "business_contact_email": business_contact_email,
                    "business_contact_first_name": business_contact_first_name,
                    "business_contact_last_name": business_contact_last_name,
                    "business_contact_phone": business_contact_phone,
                    "business_name": business_name,
                    "business_state": business_state,
                    "business_zip": business_zip,
                    "corporate_website": corporate_website,
                    "message_volume": message_volume,
                    "opt_in_workflow": opt_in_workflow,
                    "opt_in_workflow_image_urls": opt_in_workflow_image_urls,
                    "phone_numbers": phone_numbers,
                    "production_message_content": production_message_content,
                    "use_case": use_case,
                    "use_case_summary": use_case_summary,
                    "age_gated_content": age_gated_content,
                    "business_addr2": business_addr2,
                    "business_registration_country": business_registration_country,
                    "business_registration_number": business_registration_number,
                    "business_registration_type": business_registration_type,
                    "campaign_verify_authorization_token": campaign_verify_authorization_token,
                    "doing_business_as": doing_business_as,
                    "entity_type": entity_type,
                    "help_message_response": help_message_response,
                    "isv_reseller": isv_reseller,
                    "opt_in_confirmation_response": opt_in_confirmation_response,
                    "opt_in_keywords": opt_in_keywords,
                    "privacy_policy_url": privacy_policy_url,
                    "terms_and_condition_url": terms_and_condition_url,
                    "webhook_url": webhook_url,
                },
                request_create_params.RequestCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRequestEgress,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRequestStatus:
        """
        Get a single verification request by its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/messaging_tollfree/verification/requests/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRequestStatus,
        )

    def update(
        self,
        id: str,
        *,
        additional_information: str,
        business_addr1: str,
        business_city: str,
        business_contact_email: str,
        business_contact_first_name: str,
        business_contact_last_name: str,
        business_contact_phone: str,
        business_name: str,
        business_state: str,
        business_zip: str,
        corporate_website: str,
        message_volume: Volume,
        opt_in_workflow: str,
        opt_in_workflow_image_urls: Iterable[URLParam],
        phone_numbers: Iterable[TfPhoneNumberParam],
        production_message_content: str,
        use_case: UseCaseCategories,
        use_case_summary: str,
        age_gated_content: bool | Omit = omit,
        business_addr2: str | Omit = omit,
        business_registration_country: Optional[str] | Omit = omit,
        business_registration_number: Optional[str] | Omit = omit,
        business_registration_type: Optional[str] | Omit = omit,
        campaign_verify_authorization_token: Optional[str] | Omit = omit,
        doing_business_as: Optional[str] | Omit = omit,
        entity_type: Optional[TollFreeVerificationEntityType] | Omit = omit,
        help_message_response: Optional[str] | Omit = omit,
        isv_reseller: Optional[str] | Omit = omit,
        opt_in_confirmation_response: Optional[str] | Omit = omit,
        opt_in_keywords: Optional[str] | Omit = omit,
        privacy_policy_url: Optional[str] | Omit = omit,
        terms_and_condition_url: Optional[str] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRequestEgress:
        """Update an existing tollfree verification request.

        This is particularly useful
        when there are pending customer actions to be taken.

        Args:
          additional_information: Any additional information

          business_addr1: Line 1 of the business address

          business_city: The city of the business address; the first letter should be capitalized

          business_contact_email: The email address of the business contact

          business_contact_first_name: First name of the business contact; there are no specific requirements on
              formatting

          business_contact_last_name: Last name of the business contact; there are no specific requirements on
              formatting

          business_contact_phone: The phone number of the business contact in E.164 format

          business_name: Name of the business; there are no specific formatting requirements

          business_state: The full name of the state (not the 2 letter code) of the business address; the
              first letter should be capitalized

          business_zip: The ZIP code of the business address

          corporate_website: A URL, including the scheme, pointing to the corporate website

          message_volume: Message Volume Enums

          opt_in_workflow: Human-readable description of how end users will opt into receiving messages
              from the given phone numbers

          opt_in_workflow_image_urls: Images showing the opt-in workflow

          phone_numbers: The phone numbers to request the verification of

          production_message_content: An example of a message that will be sent from the given phone numbers

          use_case: Tollfree usecase categories

          use_case_summary: Human-readable summary of the desired use-case

          age_gated_content: Indicates if messaging content requires age gating (e.g., 18+). Defaults to
              false if not provided.

          business_addr2: Line 2 of the business address

          business_registration_country: ISO 3166-1 alpha-2 country code of the issuing business authority. Must be
              exactly 2 letters. Automatically converted to uppercase. Required from
              January 2026.

          business_registration_number: Official business registration number (e.g., Employer Identification Number
              (EIN) in the U.S.). Required from January 2026.

          business_registration_type: Type of business registration being provided. Required from January 2026.

          campaign_verify_authorization_token: Campaign Verify Authorization Token required for Political use case submissions
              starting February 17, 2026. This token is validated by Zipwhip and must be
              provided for all Political use case verifications after the deadline.

          doing_business_as: Doing Business As (DBA) name if different from legal name

          entity_type: Business entity classification

          help_message_response: The message returned when users text 'HELP'

          isv_reseller: ISV name

          opt_in_confirmation_response: Message sent to users confirming their opt-in to receive messages

          opt_in_keywords: Keywords used to collect and process consumer opt-ins

          privacy_policy_url: URL pointing to the business's privacy policy. Plain string, no URL format
              validation.

          terms_and_condition_url: URL pointing to the business's terms and conditions. Plain string, no URL format
              validation.

          webhook_url: URL that should receive webhooks relating to this verification request

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._patch(
            f"/messaging_tollfree/verification/requests/{id}",
            body=maybe_transform(
                {
                    "additional_information": additional_information,
                    "business_addr1": business_addr1,
                    "business_city": business_city,
                    "business_contact_email": business_contact_email,
                    "business_contact_first_name": business_contact_first_name,
                    "business_contact_last_name": business_contact_last_name,
                    "business_contact_phone": business_contact_phone,
                    "business_name": business_name,
                    "business_state": business_state,
                    "business_zip": business_zip,
                    "corporate_website": corporate_website,
                    "message_volume": message_volume,
                    "opt_in_workflow": opt_in_workflow,
                    "opt_in_workflow_image_urls": opt_in_workflow_image_urls,
                    "phone_numbers": phone_numbers,
                    "production_message_content": production_message_content,
                    "use_case": use_case,
                    "use_case_summary": use_case_summary,
                    "age_gated_content": age_gated_content,
                    "business_addr2": business_addr2,
                    "business_registration_country": business_registration_country,
                    "business_registration_number": business_registration_number,
                    "business_registration_type": business_registration_type,
                    "campaign_verify_authorization_token": campaign_verify_authorization_token,
                    "doing_business_as": doing_business_as,
                    "entity_type": entity_type,
                    "help_message_response": help_message_response,
                    "isv_reseller": isv_reseller,
                    "opt_in_confirmation_response": opt_in_confirmation_response,
                    "opt_in_keywords": opt_in_keywords,
                    "privacy_policy_url": privacy_policy_url,
                    "terms_and_condition_url": terms_and_condition_url,
                    "webhook_url": webhook_url,
                },
                request_update_params.RequestUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRequestEgress,
        )

    def list(
        self,
        *,
        page: int,
        page_size: int,
        business_name: str | Omit = omit,
        date_end: Union[str, datetime] | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        phone_number: str | Omit = omit,
        status: TfVerificationStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultPaginationForMessagingTollfree[VerificationRequestStatus]:
        """
        Get a list of previously-submitted tollfree verification requests

        Args:
          page_size: Request this many records per page

                      This value is automatically clamped if the provided value is too large.

          business_name: Filter verification requests by business name

          status: Tollfree verification status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/messaging_tollfree/verification/requests",
            page=SyncDefaultPaginationForMessagingTollfree[VerificationRequestStatus],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "page_size": page_size,
                        "business_name": business_name,
                        "date_end": date_end,
                        "date_start": date_start,
                        "phone_number": phone_number,
                        "status": status,
                    },
                    request_list_params.RequestListParams,
                ),
            ),
            model=VerificationRequestStatus,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a verification request

        A request may only be deleted when when the request is in the "rejected" state.

        - `HTTP 200`: request successfully deleted
        - `HTTP 400`: request exists but can't be deleted (i.e. not rejected)
        - `HTTP 404`: request unknown or already deleted

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/messaging_tollfree/verification/requests/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def retrieve_status_history(
        self,
        id: str,
        *,
        page_number: int,
        page_size: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequestRetrieveStatusHistoryResponse:
        """
        Get the history of status changes for a verification request.

        Returns a paginated list of historical status changes including the reason for
        each change and when it occurred.

        Args:
          page_size: Request this many records per page. This value is automatically clamped if the
              provided value is too large.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/messaging_tollfree/verification/requests/{id}/status_history",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    request_retrieve_status_history_params.RequestRetrieveStatusHistoryParams,
                ),
            ),
            cast_to=RequestRetrieveStatusHistoryResponse,
        )


class AsyncRequestsResource(AsyncAPIResource):
    """Manage your tollfree verification requests"""

    @cached_property
    def with_raw_response(self) -> AsyncRequestsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncRequestsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncRequestsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncRequestsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        additional_information: str,
        business_addr1: str,
        business_city: str,
        business_contact_email: str,
        business_contact_first_name: str,
        business_contact_last_name: str,
        business_contact_phone: str,
        business_name: str,
        business_state: str,
        business_zip: str,
        corporate_website: str,
        message_volume: Volume,
        opt_in_workflow: str,
        opt_in_workflow_image_urls: Iterable[URLParam],
        phone_numbers: Iterable[TfPhoneNumberParam],
        production_message_content: str,
        use_case: UseCaseCategories,
        use_case_summary: str,
        age_gated_content: bool | Omit = omit,
        business_addr2: str | Omit = omit,
        business_registration_country: Optional[str] | Omit = omit,
        business_registration_number: Optional[str] | Omit = omit,
        business_registration_type: Optional[str] | Omit = omit,
        campaign_verify_authorization_token: Optional[str] | Omit = omit,
        doing_business_as: Optional[str] | Omit = omit,
        entity_type: Optional[TollFreeVerificationEntityType] | Omit = omit,
        help_message_response: Optional[str] | Omit = omit,
        isv_reseller: Optional[str] | Omit = omit,
        opt_in_confirmation_response: Optional[str] | Omit = omit,
        opt_in_keywords: Optional[str] | Omit = omit,
        privacy_policy_url: Optional[str] | Omit = omit,
        terms_and_condition_url: Optional[str] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRequestEgress:
        """
        Submit a new tollfree verification request

        Args:
          additional_information: Any additional information

          business_addr1: Line 1 of the business address

          business_city: The city of the business address; the first letter should be capitalized

          business_contact_email: The email address of the business contact

          business_contact_first_name: First name of the business contact; there are no specific requirements on
              formatting

          business_contact_last_name: Last name of the business contact; there are no specific requirements on
              formatting

          business_contact_phone: The phone number of the business contact in E.164 format

          business_name: Name of the business; there are no specific formatting requirements

          business_state: The full name of the state (not the 2 letter code) of the business address; the
              first letter should be capitalized

          business_zip: The ZIP code of the business address

          corporate_website: A URL, including the scheme, pointing to the corporate website

          message_volume: Message Volume Enums

          opt_in_workflow: Human-readable description of how end users will opt into receiving messages
              from the given phone numbers

          opt_in_workflow_image_urls: Images showing the opt-in workflow

          phone_numbers: The phone numbers to request the verification of

          production_message_content: An example of a message that will be sent from the given phone numbers

          use_case: Tollfree usecase categories

          use_case_summary: Human-readable summary of the desired use-case

          age_gated_content: Indicates if messaging content requires age gating (e.g., 18+). Defaults to
              false if not provided.

          business_addr2: Line 2 of the business address

          business_registration_country: ISO 3166-1 alpha-2 country code of the issuing business authority. Must be
              exactly 2 letters. Automatically converted to uppercase. Required from
              January 2026.

          business_registration_number: Official business registration number (e.g., Employer Identification Number
              (EIN) in the U.S.). Required from January 2026.

          business_registration_type: Type of business registration being provided. Required from January 2026.

          campaign_verify_authorization_token: Campaign Verify Authorization Token required for Political use case submissions
              starting February 17, 2026. This token is validated by Zipwhip and must be
              provided for all Political use case verifications after the deadline.

          doing_business_as: Doing Business As (DBA) name if different from legal name

          entity_type: Business entity classification

          help_message_response: The message returned when users text 'HELP'

          isv_reseller: ISV name

          opt_in_confirmation_response: Message sent to users confirming their opt-in to receive messages

          opt_in_keywords: Keywords used to collect and process consumer opt-ins

          privacy_policy_url: URL pointing to the business's privacy policy. Plain string, no URL format
              validation.

          terms_and_condition_url: URL pointing to the business's terms and conditions. Plain string, no URL format
              validation.

          webhook_url: URL that should receive webhooks relating to this verification request

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/messaging_tollfree/verification/requests",
            body=await async_maybe_transform(
                {
                    "additional_information": additional_information,
                    "business_addr1": business_addr1,
                    "business_city": business_city,
                    "business_contact_email": business_contact_email,
                    "business_contact_first_name": business_contact_first_name,
                    "business_contact_last_name": business_contact_last_name,
                    "business_contact_phone": business_contact_phone,
                    "business_name": business_name,
                    "business_state": business_state,
                    "business_zip": business_zip,
                    "corporate_website": corporate_website,
                    "message_volume": message_volume,
                    "opt_in_workflow": opt_in_workflow,
                    "opt_in_workflow_image_urls": opt_in_workflow_image_urls,
                    "phone_numbers": phone_numbers,
                    "production_message_content": production_message_content,
                    "use_case": use_case,
                    "use_case_summary": use_case_summary,
                    "age_gated_content": age_gated_content,
                    "business_addr2": business_addr2,
                    "business_registration_country": business_registration_country,
                    "business_registration_number": business_registration_number,
                    "business_registration_type": business_registration_type,
                    "campaign_verify_authorization_token": campaign_verify_authorization_token,
                    "doing_business_as": doing_business_as,
                    "entity_type": entity_type,
                    "help_message_response": help_message_response,
                    "isv_reseller": isv_reseller,
                    "opt_in_confirmation_response": opt_in_confirmation_response,
                    "opt_in_keywords": opt_in_keywords,
                    "privacy_policy_url": privacy_policy_url,
                    "terms_and_condition_url": terms_and_condition_url,
                    "webhook_url": webhook_url,
                },
                request_create_params.RequestCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRequestEgress,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRequestStatus:
        """
        Get a single verification request by its ID.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/messaging_tollfree/verification/requests/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRequestStatus,
        )

    async def update(
        self,
        id: str,
        *,
        additional_information: str,
        business_addr1: str,
        business_city: str,
        business_contact_email: str,
        business_contact_first_name: str,
        business_contact_last_name: str,
        business_contact_phone: str,
        business_name: str,
        business_state: str,
        business_zip: str,
        corporate_website: str,
        message_volume: Volume,
        opt_in_workflow: str,
        opt_in_workflow_image_urls: Iterable[URLParam],
        phone_numbers: Iterable[TfPhoneNumberParam],
        production_message_content: str,
        use_case: UseCaseCategories,
        use_case_summary: str,
        age_gated_content: bool | Omit = omit,
        business_addr2: str | Omit = omit,
        business_registration_country: Optional[str] | Omit = omit,
        business_registration_number: Optional[str] | Omit = omit,
        business_registration_type: Optional[str] | Omit = omit,
        campaign_verify_authorization_token: Optional[str] | Omit = omit,
        doing_business_as: Optional[str] | Omit = omit,
        entity_type: Optional[TollFreeVerificationEntityType] | Omit = omit,
        help_message_response: Optional[str] | Omit = omit,
        isv_reseller: Optional[str] | Omit = omit,
        opt_in_confirmation_response: Optional[str] | Omit = omit,
        opt_in_keywords: Optional[str] | Omit = omit,
        privacy_policy_url: Optional[str] | Omit = omit,
        terms_and_condition_url: Optional[str] | Omit = omit,
        webhook_url: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> VerificationRequestEgress:
        """Update an existing tollfree verification request.

        This is particularly useful
        when there are pending customer actions to be taken.

        Args:
          additional_information: Any additional information

          business_addr1: Line 1 of the business address

          business_city: The city of the business address; the first letter should be capitalized

          business_contact_email: The email address of the business contact

          business_contact_first_name: First name of the business contact; there are no specific requirements on
              formatting

          business_contact_last_name: Last name of the business contact; there are no specific requirements on
              formatting

          business_contact_phone: The phone number of the business contact in E.164 format

          business_name: Name of the business; there are no specific formatting requirements

          business_state: The full name of the state (not the 2 letter code) of the business address; the
              first letter should be capitalized

          business_zip: The ZIP code of the business address

          corporate_website: A URL, including the scheme, pointing to the corporate website

          message_volume: Message Volume Enums

          opt_in_workflow: Human-readable description of how end users will opt into receiving messages
              from the given phone numbers

          opt_in_workflow_image_urls: Images showing the opt-in workflow

          phone_numbers: The phone numbers to request the verification of

          production_message_content: An example of a message that will be sent from the given phone numbers

          use_case: Tollfree usecase categories

          use_case_summary: Human-readable summary of the desired use-case

          age_gated_content: Indicates if messaging content requires age gating (e.g., 18+). Defaults to
              false if not provided.

          business_addr2: Line 2 of the business address

          business_registration_country: ISO 3166-1 alpha-2 country code of the issuing business authority. Must be
              exactly 2 letters. Automatically converted to uppercase. Required from
              January 2026.

          business_registration_number: Official business registration number (e.g., Employer Identification Number
              (EIN) in the U.S.). Required from January 2026.

          business_registration_type: Type of business registration being provided. Required from January 2026.

          campaign_verify_authorization_token: Campaign Verify Authorization Token required for Political use case submissions
              starting February 17, 2026. This token is validated by Zipwhip and must be
              provided for all Political use case verifications after the deadline.

          doing_business_as: Doing Business As (DBA) name if different from legal name

          entity_type: Business entity classification

          help_message_response: The message returned when users text 'HELP'

          isv_reseller: ISV name

          opt_in_confirmation_response: Message sent to users confirming their opt-in to receive messages

          opt_in_keywords: Keywords used to collect and process consumer opt-ins

          privacy_policy_url: URL pointing to the business's privacy policy. Plain string, no URL format
              validation.

          terms_and_condition_url: URL pointing to the business's terms and conditions. Plain string, no URL format
              validation.

          webhook_url: URL that should receive webhooks relating to this verification request

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._patch(
            f"/messaging_tollfree/verification/requests/{id}",
            body=await async_maybe_transform(
                {
                    "additional_information": additional_information,
                    "business_addr1": business_addr1,
                    "business_city": business_city,
                    "business_contact_email": business_contact_email,
                    "business_contact_first_name": business_contact_first_name,
                    "business_contact_last_name": business_contact_last_name,
                    "business_contact_phone": business_contact_phone,
                    "business_name": business_name,
                    "business_state": business_state,
                    "business_zip": business_zip,
                    "corporate_website": corporate_website,
                    "message_volume": message_volume,
                    "opt_in_workflow": opt_in_workflow,
                    "opt_in_workflow_image_urls": opt_in_workflow_image_urls,
                    "phone_numbers": phone_numbers,
                    "production_message_content": production_message_content,
                    "use_case": use_case,
                    "use_case_summary": use_case_summary,
                    "age_gated_content": age_gated_content,
                    "business_addr2": business_addr2,
                    "business_registration_country": business_registration_country,
                    "business_registration_number": business_registration_number,
                    "business_registration_type": business_registration_type,
                    "campaign_verify_authorization_token": campaign_verify_authorization_token,
                    "doing_business_as": doing_business_as,
                    "entity_type": entity_type,
                    "help_message_response": help_message_response,
                    "isv_reseller": isv_reseller,
                    "opt_in_confirmation_response": opt_in_confirmation_response,
                    "opt_in_keywords": opt_in_keywords,
                    "privacy_policy_url": privacy_policy_url,
                    "terms_and_condition_url": terms_and_condition_url,
                    "webhook_url": webhook_url,
                },
                request_update_params.RequestUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=VerificationRequestEgress,
        )

    def list(
        self,
        *,
        page: int,
        page_size: int,
        business_name: str | Omit = omit,
        date_end: Union[str, datetime] | Omit = omit,
        date_start: Union[str, datetime] | Omit = omit,
        phone_number: str | Omit = omit,
        status: TfVerificationStatus | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[
        VerificationRequestStatus, AsyncDefaultPaginationForMessagingTollfree[VerificationRequestStatus]
    ]:
        """
        Get a list of previously-submitted tollfree verification requests

        Args:
          page_size: Request this many records per page

                      This value is automatically clamped if the provided value is too large.

          business_name: Filter verification requests by business name

          status: Tollfree verification status

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/messaging_tollfree/verification/requests",
            page=AsyncDefaultPaginationForMessagingTollfree[VerificationRequestStatus],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page": page,
                        "page_size": page_size,
                        "business_name": business_name,
                        "date_end": date_end,
                        "date_start": date_start,
                        "phone_number": phone_number,
                        "status": status,
                    },
                    request_list_params.RequestListParams,
                ),
            ),
            model=VerificationRequestStatus,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a verification request

        A request may only be deleted when when the request is in the "rejected" state.

        - `HTTP 200`: request successfully deleted
        - `HTTP 400`: request exists but can't be deleted (i.e. not rejected)
        - `HTTP 404`: request unknown or already deleted

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/messaging_tollfree/verification/requests/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def retrieve_status_history(
        self,
        id: str,
        *,
        page_number: int,
        page_size: int,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> RequestRetrieveStatusHistoryResponse:
        """
        Get the history of status changes for a verification request.

        Returns a paginated list of historical status changes including the reason for
        each change and when it occurred.

        Args:
          page_size: Request this many records per page. This value is automatically clamped if the
              provided value is too large.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/messaging_tollfree/verification/requests/{id}/status_history",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    request_retrieve_status_history_params.RequestRetrieveStatusHistoryParams,
                ),
            ),
            cast_to=RequestRetrieveStatusHistoryResponse,
        )


class RequestsResourceWithRawResponse:
    def __init__(self, requests: RequestsResource) -> None:
        self._requests = requests

        self.create = to_raw_response_wrapper(
            requests.create,
        )
        self.retrieve = to_raw_response_wrapper(
            requests.retrieve,
        )
        self.update = to_raw_response_wrapper(
            requests.update,
        )
        self.list = to_raw_response_wrapper(
            requests.list,
        )
        self.delete = to_raw_response_wrapper(
            requests.delete,
        )
        self.retrieve_status_history = to_raw_response_wrapper(
            requests.retrieve_status_history,
        )


class AsyncRequestsResourceWithRawResponse:
    def __init__(self, requests: AsyncRequestsResource) -> None:
        self._requests = requests

        self.create = async_to_raw_response_wrapper(
            requests.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            requests.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            requests.update,
        )
        self.list = async_to_raw_response_wrapper(
            requests.list,
        )
        self.delete = async_to_raw_response_wrapper(
            requests.delete,
        )
        self.retrieve_status_history = async_to_raw_response_wrapper(
            requests.retrieve_status_history,
        )


class RequestsResourceWithStreamingResponse:
    def __init__(self, requests: RequestsResource) -> None:
        self._requests = requests

        self.create = to_streamed_response_wrapper(
            requests.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            requests.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            requests.update,
        )
        self.list = to_streamed_response_wrapper(
            requests.list,
        )
        self.delete = to_streamed_response_wrapper(
            requests.delete,
        )
        self.retrieve_status_history = to_streamed_response_wrapper(
            requests.retrieve_status_history,
        )


class AsyncRequestsResourceWithStreamingResponse:
    def __init__(self, requests: AsyncRequestsResource) -> None:
        self._requests = requests

        self.create = async_to_streamed_response_wrapper(
            requests.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            requests.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            requests.update,
        )
        self.list = async_to_streamed_response_wrapper(
            requests.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            requests.delete,
        )
        self.retrieve_status_history = async_to_streamed_response_wrapper(
            requests.retrieve_status_history,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_speak_ended import CallSpeakEnded

__all__ = ["CallSpeakEndedWebhookEvent"]


class CallSpeakEndedWebhookEvent(BaseModel):
    data: Optional[CallSpeakEnded] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .conference_recording_saved import ConferenceRecordingSaved

__all__ = ["ConferenceRecordingSavedWebhookEvent"]


class ConferenceRecordingSavedWebhookEvent(BaseModel):
    data: Optional[ConferenceRecordingSaved] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .settings_param import SettingsParam

__all__ = ["AuthenticationProviderCreateParams"]


class AuthenticationProviderCreateParams(TypedDict, total=False):
    name: Required[str]
    """The name associated with the authentication provider."""

    settings: Required[SettingsParam]
    """The settings associated with the authentication provider."""

    short_name: Required[str]
    """The short name associated with the authentication provider.

    This must be unique and URL-friendly, as it's going to be part of the login URL.
    """

    active: bool
    """The active status of the authentication provider"""

    settings_url: str
    """
    The URL for the identity provider metadata file to populate the settings
    automatically. If the settings attribute is provided, that will be used instead.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["DetailRecordsReportCreateParams"]


class DetailRecordsReportCreateParams(TypedDict, total=False):
    end_time: str
    """ISO 8601 formatted date-time indicating the end time."""

    start_time: str
    """ISO 8601 formatted date-time indicating the start time."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .actions import (
    ActionsResource,
    AsyncActionsResource,
    ActionsResourceWithRawResponse,
    AsyncActionsResourceWithRawResponse,
    ActionsResourceWithStreamingResponse,
    AsyncActionsResourceWithStreamingResponse,
)
from .sim_cards import (
    SimCardsResource,
    AsyncSimCardsResource,
    SimCardsResourceWithRawResponse,
    AsyncSimCardsResourceWithRawResponse,
    SimCardsResourceWithStreamingResponse,
    AsyncSimCardsResourceWithStreamingResponse,
)

__all__ = [
    "ActionsResource",
    "AsyncActionsResource",
    "ActionsResourceWithRawResponse",
    "AsyncActionsResourceWithRawResponse",
    "ActionsResourceWithStreamingResponse",
    "AsyncActionsResourceWithStreamingResponse",
    "SimCardsResource",
    "AsyncSimCardsResource",
    "SimCardsResourceWithRawResponse",
    "AsyncSimCardsResourceWithRawResponse",
    "SimCardsResourceWithStreamingResponse",
    "AsyncSimCardsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Annotated, TypeAlias, TypedDict

from .._utils import PropertyInfo
from .available_service import AvailableService

__all__ = [
    "NetworkCoverageListParams",
    "Filter",
    "Filters",
    "FiltersAvailableServices",
    "FiltersAvailableServicesContains",
]


class NetworkCoverageListParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[location.region], filter[location.site],
    filter[location.pop], filter[location.code]
    """

    filters: Filters
    """Consolidated filters parameter (deepObject style).

    Originally: filters[available_services][contains]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[location.region], filter[location.site], filter[location.pop], filter[location.code]
    """

    location_code: Annotated[str, PropertyInfo(alias="location.code")]
    """The code of associated location to filter on."""

    location_pop: Annotated[str, PropertyInfo(alias="location.pop")]
    """The POP of associated location to filter on."""

    location_region: Annotated[str, PropertyInfo(alias="location.region")]
    """The region of associated location to filter on."""

    location_site: Annotated[str, PropertyInfo(alias="location.site")]
    """The site of associated location to filter on."""


class FiltersAvailableServicesContains(TypedDict, total=False):
    """Available service filtering operations"""

    contains: AvailableService
    """Filter by available services containing the specified service"""


FiltersAvailableServices: TypeAlias = Union[AvailableService, FiltersAvailableServicesContains]


class Filters(TypedDict, total=False):
    """Consolidated filters parameter (deepObject style).

    Originally: filters[available_services][contains]
    """

    available_services: FiltersAvailableServices
    """Filter by exact available service match"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from .._models import BaseModel
from .sub_number_order_regulatory_requirement import SubNumberOrderRegulatoryRequirement

__all__ = ["SubNumberOrder"]


class SubNumberOrder(BaseModel):
    id: Optional[str] = None

    country_code: Optional[str] = None

    created_at: Optional[datetime] = None
    """An ISO 8901 datetime string denoting when the number order was created."""

    customer_reference: Optional[str] = None
    """A customer reference string for customer look ups."""

    is_block_sub_number_order: Optional[bool] = None
    """True if the sub number order is a block sub number order"""

    order_request_id: Optional[str] = None

    phone_number_type: Optional[Literal["local", "toll_free", "mobile", "national", "shared_cost", "landline"]] = None

    phone_numbers_count: Optional[int] = None
    """The count of phone numbers in the number order."""

    record_type: Optional[str] = None

    regulatory_requirements: Optional[List[SubNumberOrderRegulatoryRequirement]] = None

    requirements_met: Optional[bool] = None
    """True if all requirements are met for every phone number, false otherwise."""

    status: Optional[Literal["pending", "success", "failure"]] = None
    """The status of the order."""

    updated_at: Optional[datetime] = None
    """An ISO 8901 datetime string for when the number order was updated."""

    user_id: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ActionStartSiprecParams"]


class ActionStartSiprecParams(TypedDict, total=False):
    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    connector_name: str
    """Name of configured SIPREC connector to be used."""

    include_metadata_custom_headers: Literal[True, False]
    """
    When set, custom parameters will be added as metadata
    (recording.session.ExtensionParameters). Otherwise, they’ll be added to sip
    headers.
    """

    secure: Literal[True, False]
    """Controls whether to encrypt media sent to your SRS using SRTP and TLS.

    When set you need to configure SRS port in your connector to 5061.
    """

    session_timeout_secs: int
    """Sets `Session-Expires` header to the INVITE.

    A reinvite is sent every half the value set. Usefull for session keep alive.
    Minimum value is 90, set to 0 to disable.
    """

    sip_transport: Literal["udp", "tcp", "tls"]
    """Specifies SIP transport protocol."""

    siprec_track: Literal["inbound_track", "outbound_track", "both_tracks"]
    """Specifies which track should be sent on siprec session."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import json
import time
import base64
from typing import Mapping, cast

from .._models import construct_type
from .._resource import SyncAPIResource, AsyncAPIResource
from .._exceptions import TelnyxError
from ..types.unwrap_webhook_event import UnwrapWebhookEvent
from ..types.unsafe_unwrap_webhook_event import UnsafeUnwrapWebhookEvent

__all__ = ["WebhooksResource", "AsyncWebhooksResource", "TelnyxWebhookVerificationError"]


class TelnyxWebhookVerificationError(TelnyxError):
    """Raised when webhook signature verification fails."""
    pass


def _get_header_case_insensitive(headers: dict[str, str], key: str) -> str | None:
    """Get header value case-insensitively."""
    key_lower = key.lower()
    for header_key, header_value in headers.items():
        if header_key.lower() == key_lower:
            return header_value
    return None


def _verify_signature(payload: str, headers: dict[str, str], public_key: str | bytes) -> None:
    """
    Verify Telnyx webhook signature using ED25519 cryptography.

    Args:
        payload: The raw webhook payload as a string
        headers: The webhook headers
        public_key: The ED25519 public key (base64 string from Mission Control)

    Raises:
        TelnyxWebhookVerificationError: If verification fails
    """
    try:
        from nacl.signing import VerifyKey
        from nacl.exceptions import BadSignatureError
    except ImportError as exc:
        raise TelnyxError("You need to install `pynacl` to use webhook verification") from exc

    # Extract required headers (case-insensitive)
    signature_header = _get_header_case_insensitive(headers, 'telnyx-signature-ed25519')
    timestamp_header = _get_header_case_insensitive(headers, 'telnyx-timestamp')

    if not signature_header:
        raise TelnyxWebhookVerificationError("Missing header: Telnyx-Signature-Ed25519")

    if not timestamp_header:
        raise TelnyxWebhookVerificationError("Missing header: Telnyx-Timestamp")

    # Validate timestamp to prevent replay attacks (5 minute tolerance)
    try:
        webhook_time = int(timestamp_header)
        current_time = int(time.time())
        if abs(current_time - webhook_time) > 300:  # 5 minutes
            raise TelnyxWebhookVerificationError(f"Webhook timestamp too old or too new: {timestamp_header}")
    except ValueError as exc:
        raise TelnyxWebhookVerificationError(f"Invalid timestamp format: {timestamp_header}") from exc

    # Decode public key from base64 (Telnyx provides keys in base64 format)
    try:
        if isinstance(public_key, str):
            public_key_bytes = base64.b64decode(public_key)
        else:
            public_key_bytes = public_key

        if len(public_key_bytes) != 32:
            raise TelnyxWebhookVerificationError(
                f"Invalid public key: expected 32 bytes, got {len(public_key_bytes)} bytes"
            )

        verify_key = VerifyKey(public_key_bytes)
    except Exception as exc:
        raise TelnyxWebhookVerificationError(f"Invalid public key format: {exc}") from exc

    # Decode signature from base64 (Telnyx sends signatures in base64 format)
    try:
        signature_bytes = base64.b64decode(signature_header)

        if len(signature_bytes) != 64:
            raise TelnyxWebhookVerificationError(
                f"Invalid signature length: expected 64 bytes, got {len(signature_bytes)} bytes"
            )
    except Exception as exc:
        raise TelnyxWebhookVerificationError(f"Invalid signature format: {exc}") from exc

    # Create the signed payload: timestamp|payload
    signed_payload = f"{timestamp_header}|{payload}".encode('utf-8')

    # Verify the signature
    try:
        verify_key.verify(signed_payload, signature_bytes)
    except BadSignatureError as exc:
        raise TelnyxWebhookVerificationError(
            "Signature verification failed: signature does not match payload"
        ) from exc


class WebhooksResource(SyncAPIResource):
    def unsafe_unwrap(self, payload: str) -> UnsafeUnwrapWebhookEvent:
        return cast(
            UnsafeUnwrapWebhookEvent,
            construct_type(
                type_=UnsafeUnwrapWebhookEvent,
                value=json.loads(payload),
            ),
        )

    def unwrap(self, payload: str, *, headers: Mapping[str, str], key: str | bytes | None = None) -> UnwrapWebhookEvent:
        if key is None:
            key = self._client.public_key
            if key is None:
                raise ValueError(
                    "Cannot verify a webhook without a key on either the client's public_key or passed in as an argument"
                )

        if not isinstance(headers, dict):
            headers = dict(headers)

        _verify_signature(payload, headers, key)

        return cast(
            UnwrapWebhookEvent,
            construct_type(
                type_=UnwrapWebhookEvent,
                value=json.loads(payload),
            ),
        )


class AsyncWebhooksResource(AsyncAPIResource):
    def unsafe_unwrap(self, payload: str) -> UnsafeUnwrapWebhookEvent:
        return cast(
            UnsafeUnwrapWebhookEvent,
            construct_type(
                type_=UnsafeUnwrapWebhookEvent,
                value=json.loads(payload),
            ),
        )

    def unwrap(self, payload: str, *, headers: Mapping[str, str], key: str | bytes | None = None) -> UnwrapWebhookEvent:
        if key is None:
            key = self._client.public_key
            if key is None:
                raise ValueError(
                    "Cannot verify a webhook without a key on either the client's public_key or passed in as an argument"
                )

        if not isinstance(headers, dict):
            headers = dict(headers)

        _verify_signature(payload, headers, key)

        return cast(
            UnwrapWebhookEvent,
            construct_type(
                type_=UnwrapWebhookEvent,
                value=json.loads(payload),
            ),
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["BrandGetFeedbackResponse", "Category"]


class Category(BaseModel):
    id: str
    """One of `TAX_ID`, `STOCK_SYMBOL`, `GOVERNMENT_ENTITY`, `NONPROFIT`, and `OTHERS`"""

    description: str
    """Long-form description of the feedback with additional information"""

    display_name: str = FieldInfo(alias="displayName")
    """Human-readable version of the `id` field"""

    fields: List[str]
    """List of relevant fields in the originally-submitted brand json"""


class BrandGetFeedbackResponse(BaseModel):
    brand_id: str = FieldInfo(alias="brandId")
    """ID of the brand being queried about"""

    category: List[Category]
    """A list of reasons why brand creation/revetting didn't go as planned"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, List, Optional

from .._models import BaseModel

__all__ = ["EventNode", "Cost", "Links", "Relationship", "RelationshipVia"]


class Cost(BaseModel):
    cumulative_cost: str
    """Cumulative cost including all descendants."""

    currency: str
    """ISO 4217 currency code."""

    event_cost: str
    """Cost of this individual event."""


class Links(BaseModel):
    records: str
    """Link to the underlying detail records."""

    self: str
    """Link to this session analysis node."""


class RelationshipVia(BaseModel):
    local_field: str
    """Field name on the child record."""

    parent_field: str
    """Field name on the parent record."""


class Relationship(BaseModel):
    """Relationship to the parent node, null for root."""

    parent_id: str
    """Identifier of the parent event."""

    type: str
    """Relationship type identifier."""

    via: RelationshipVia


class EventNode(BaseModel):
    id: str
    """Event identifier."""

    children: List["EventNode"]
    """Child events in the session tree."""

    cost: Cost

    event_name: str
    """Name of the event type."""

    links: Links

    product: str
    """Product that generated this event."""

    record: Dict[str, object]
    """The underlying detail record data. Contents vary by record type."""

    relationship: Optional[Relationship] = None
    """Relationship to the parent node, null for root."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .template_list_params import TemplateListParams as TemplateListParams
from .template_create_params import TemplateCreateParams as TemplateCreateParams
from .phone_number_list_params import PhoneNumberListParams as PhoneNumberListParams
from .template_create_response import TemplateCreateResponse as TemplateCreateResponse
from .phone_number_list_response import PhoneNumberListResponse as PhoneNumberListResponse
from .phone_number_verify_params import PhoneNumberVerifyParams as PhoneNumberVerifyParams
from .business_account_list_params import BusinessAccountListParams as BusinessAccountListParams
from .business_account_list_response import BusinessAccountListResponse as BusinessAccountListResponse
from .business_account_retrieve_response import BusinessAccountRetrieveResponse as BusinessAccountRetrieveResponse
from .phone_number_resend_verification_params import (
    PhoneNumberResendVerificationParams as PhoneNumberResendVerificationParams,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .texml_application import TexmlApplication

__all__ = ["TexmlApplicationDeleteResponse"]


class TexmlApplicationDeleteResponse(BaseModel):
    data: Optional[TexmlApplication] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Annotated, TypedDict

from .._utils import PropertyInfo

__all__ = ["MobileVoiceConnectionListParams"]


class MobileVoiceConnectionListParams(TypedDict, total=False):
    filter_connection_name_contains: Annotated[str, PropertyInfo(alias="filter[connection_name][contains]")]
    """Filter by connection name containing the given string"""

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]
    """The page number to load"""

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]
    """The size of the page"""

    sort: str
    """Sort by field (e.g., created_at, connection_name, active).

    Prefix with - for descending order.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ResembleVoiceSettings"]


class ResembleVoiceSettings(BaseModel):
    type: Literal["resemble"]
    """Voice settings provider type"""

    format: Optional[Literal["wav", "mp3"]] = None
    """Output audio format."""

    precision: Optional[Literal["PCM_16", "PCM_24", "PCM_32", "MULAW"]] = None
    """Audio precision format."""

    sample_rate: Optional[Literal["8000", "16000", "22050", "32000", "44100", "48000"]] = None
    """Audio sample rate in Hz."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["GlobalIPAssignment"]


class GlobalIPAssignment(BaseModel):
    id: Optional[str] = None
    """Identifies the resource."""

    created_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was created."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[str] = None
    """ISO 8601 formatted date-time indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_machine_greeting_ended import CallMachineGreetingEnded

__all__ = ["CallMachineGreetingEndedWebhookEvent"]


class CallMachineGreetingEndedWebhookEvent(BaseModel):
    data: Optional[CallMachineGreetingEnded] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["DynamicEmergencyAddressCreateParams"]


class DynamicEmergencyAddressCreateParams(TypedDict, total=False):
    administrative_area: Required[str]

    country_code: Required[Literal["US", "CA", "PR"]]

    house_number: Required[str]

    locality: Required[str]

    postal_code: Required[str]

    street_name: Required[str]

    extended_address: str

    house_suffix: str

    street_post_directional: str

    street_pre_directional: str

    street_suffix: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .migration_params import MigrationParams as MigrationParams
from .migration_create_params import MigrationCreateParams as MigrationCreateParams
from .migration_list_response import MigrationListResponse as MigrationListResponse
from .migration_source_params import MigrationSourceParams as MigrationSourceParams
from .migration_create_response import MigrationCreateResponse as MigrationCreateResponse
from .migration_retrieve_response import MigrationRetrieveResponse as MigrationRetrieveResponse
from .migration_source_create_params import MigrationSourceCreateParams as MigrationSourceCreateParams
from .migration_source_list_response import MigrationSourceListResponse as MigrationSourceListResponse
from .migration_source_create_response import MigrationSourceCreateResponse as MigrationSourceCreateResponse
from .migration_source_delete_response import MigrationSourceDeleteResponse as MigrationSourceDeleteResponse
from .bucket_create_presigned_url_params import BucketCreatePresignedURLParams as BucketCreatePresignedURLParams
from .migration_source_retrieve_response import MigrationSourceRetrieveResponse as MigrationSourceRetrieveResponse
from .bucket_create_presigned_url_response import BucketCreatePresignedURLResponse as BucketCreatePresignedURLResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .conference_ended import ConferenceEnded

__all__ = ["ConferenceEndedWebhookEvent"]


class ConferenceEndedWebhookEvent(BaseModel):
    data: Optional[ConferenceEnded] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["MdrUsageReport", "Result"]


class Result(BaseModel):
    carrier_passthrough_fee: Optional[str] = None

    connection: Optional[str] = None

    cost: Optional[str] = None

    currency: Optional[str] = None

    delivered: Optional[str] = None

    direction: Optional[str] = None

    message_type: Optional[str] = None

    parts: Optional[str] = None

    product: Optional[str] = None

    profile_id: Optional[str] = None

    received: Optional[str] = None

    sent: Optional[str] = None

    tags: Optional[str] = None

    tn_type: Optional[str] = None


class MdrUsageReport(BaseModel):
    id: Optional[str] = None
    """Identifies the resource"""

    aggregation_type: Optional[Literal["NO_AGGREGATION", "PROFILE", "TAGS"]] = None

    connections: Optional[List[int]] = None

    created_at: Optional[datetime] = None

    end_date: Optional[datetime] = None

    profiles: Optional[str] = None

    record_type: Optional[str] = None

    report_url: Optional[str] = None

    result: Optional[List[Result]] = None

    start_date: Optional[datetime] = None

    status: Optional[Literal["PENDING", "COMPLETE", "FAILED", "EXPIRED"]] = None

    updated_at: Optional[datetime] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["PaginationMetaCloudflareIPListSync"]


class PaginationMetaCloudflareIPListSync(BaseModel):
    page_number: int

    page_size: int

    total_pages: int

    total_results: int



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RegionInformation"]


class RegionInformation(BaseModel):
    region_name: Optional[str] = None

    region_type: Optional[Literal["country_code", "rate_center", "state", "location"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Literal, Required, TypedDict

__all__ = ["EventLogParams"]


class EventLogParams(TypedDict, total=False):
    mission_id: Required[str]

    summary: Required[str]

    type: Required[
        Literal[
            "status_change",
            "step_started",
            "step_completed",
            "step_failed",
            "tool_call",
            "tool_result",
            "message",
            "error",
            "custom",
        ]
    ]

    agent_id: str

    idempotency_key: str
    """Prevents duplicate events on retry"""

    payload: Dict[str, object]

    step_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from datetime import date
from typing_extensions import Annotated, TypedDict

from ..._utils import PropertyInfo

__all__ = [
    "SessionRetrieveParticipantsParams",
    "Filter",
    "FilterDateJoinedAt",
    "FilterDateLeftAt",
    "FilterDateUpdatedAt",
]


class SessionRetrieveParticipantsParams(TypedDict, total=False):
    filter: Filter
    """Consolidated filter parameter (deepObject style).

    Originally: filter[date_joined_at][eq], filter[date_joined_at][gte],
    filter[date_joined_at][lte], filter[date_updated_at][eq],
    filter[date_updated_at][gte], filter[date_updated_at][lte],
    filter[date_left_at][eq], filter[date_left_at][gte], filter[date_left_at][lte],
    filter[context]
    """

    page_number: Annotated[int, PropertyInfo(alias="page[number]")]

    page_size: Annotated[int, PropertyInfo(alias="page[size]")]


class FilterDateJoinedAt(TypedDict, total=False):
    eq: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room participants that joined on that date."""

    gte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """
    ISO 8601 date for filtering room participants that joined on or after that date.
    """

    lte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """
    ISO 8601 date for filtering room participants that joined on or before that
    date.
    """


class FilterDateLeftAt(TypedDict, total=False):
    eq: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room participants that left on that date."""

    gte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room participants that left on or after that date."""

    lte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room participants that left on or before that date."""


class FilterDateUpdatedAt(TypedDict, total=False):
    eq: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room participants updated on that date."""

    gte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room participants updated on or after that date."""

    lte: Annotated[Union[str, date], PropertyInfo(format="iso8601")]
    """ISO 8601 date for filtering room participants updated on or before that date."""


class Filter(TypedDict, total=False):
    """Consolidated filter parameter (deepObject style).

    Originally: filter[date_joined_at][eq], filter[date_joined_at][gte], filter[date_joined_at][lte], filter[date_updated_at][eq], filter[date_updated_at][gte], filter[date_updated_at][lte], filter[date_left_at][eq], filter[date_left_at][gte], filter[date_left_at][lte], filter[context]
    """

    context: str
    """Filter room participants based on the context."""

    date_joined_at: FilterDateJoinedAt

    date_left_at: FilterDateLeftAt

    date_updated_at: FilterDateUpdatedAt



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable
from typing_extensions import Literal, TypedDict

from ..stream_codec import StreamCodec
from ..sip_header_param import SipHeaderParam
from ..custom_sip_header_param import CustomSipHeaderParam
from ..sound_modifications_param import SoundModificationsParam
from ..stream_bidirectional_mode import StreamBidirectionalMode
from ..stream_bidirectional_codec import StreamBidirectionalCodec
from ..stream_bidirectional_target_legs import StreamBidirectionalTargetLegs
from .transcription_start_request_param import TranscriptionStartRequestParam

__all__ = ["ActionAnswerParams", "WebhookRetriesPolicies"]


class ActionAnswerParams(TypedDict, total=False):
    billing_group_id: str
    """Use this field to set the Billing Group ID for the call.

    Must be a valid and existing Billing Group ID.
    """

    client_state: str
    """Use this field to add state to every subsequent webhook.

    It must be a valid Base-64 encoded string.
    """

    command_id: str
    """Use this field to avoid duplicate commands.

    Telnyx will ignore any command with the same `command_id` for the same
    `call_control_id`.
    """

    custom_headers: Iterable[CustomSipHeaderParam]
    """Custom headers to be added to the SIP INVITE response."""

    preferred_codecs: Literal["G722,PCMU,PCMA,G729,OPUS,VP8,H264"]
    """
    The list of comma-separated codecs in a preferred order for the forked media to
    be received.
    """

    record: Literal["record-from-answer"]
    """Start recording automatically after an event. Disabled by default."""

    record_channels: Literal["single", "dual"]
    """
    Defines which channel should be recorded ('single' or 'dual') when `record` is
    specified.
    """

    record_custom_file_name: str
    """The custom recording file name to be used instead of the default `call_leg_id`.

    Telnyx will still add a Unix timestamp suffix.
    """

    record_format: Literal["wav", "mp3"]
    """
    Defines the format of the recording ('wav' or 'mp3') when `record` is specified.
    """

    record_max_length: int
    """
    Defines the maximum length for the recording in seconds when `record` is
    specified. The minimum value is 0. The maximum value is 43200. The default value
    is 0 (infinite).
    """

    record_timeout_secs: int
    """
    The number of seconds that Telnyx will wait for the recording to be stopped if
    silence is detected when `record` is specified. The timer only starts when the
    speech is detected. Please note that call transcription is used to detect
    silence and the related charge will be applied. The minimum value is 0. The
    default value is 0 (infinite).
    """

    record_track: Literal["both", "inbound", "outbound"]
    """The audio track to be recorded.

    Can be either `both`, `inbound` or `outbound`. If only single track is specified
    (`inbound`, `outbound`), `channels` configuration is ignored and it will be
    recorded as mono (single channel).
    """

    record_trim: Literal["trim-silence"]
    """
    When set to `trim-silence`, silence will be removed from the beginning and end
    of the recording.
    """

    send_silence_when_idle: bool
    """Generate silence RTP packets when no transmission available."""

    sip_headers: Iterable[SipHeaderParam]
    """SIP headers to be added to the SIP INVITE response.

    Currently only User-to-User header is supported.
    """

    sound_modifications: SoundModificationsParam
    """Use this field to modify sound effects, for example adjust the pitch."""

    stream_bidirectional_codec: StreamBidirectionalCodec
    """Indicates codec for bidirectional streaming RTP payloads.

    Used only with stream_bidirectional_mode=rtp. Case sensitive.
    """

    stream_bidirectional_mode: StreamBidirectionalMode
    """Configures method of bidirectional streaming (mp3, rtp)."""

    stream_bidirectional_target_legs: StreamBidirectionalTargetLegs
    """Specifies which call legs should receive the bidirectional stream audio."""

    stream_codec: StreamCodec
    """Specifies the codec to be used for the streamed audio.

    When set to 'default' or when transcoding is not possible, the codec from the
    call will be used.
    """

    stream_track: Literal["inbound_track", "outbound_track", "both_tracks"]
    """Specifies which track should be streamed."""

    stream_url: str
    """The destination WebSocket address where the stream is going to be delivered."""

    transcription: bool
    """Enable transcription upon call answer. The default value is false."""

    transcription_config: TranscriptionStartRequestParam

    webhook_retries_policies: Dict[str, WebhookRetriesPolicies]
    """A map of event types to retry policies.

    Each retry policy contains an array of `retries_ms` specifying the delays
    between retry attempts in milliseconds. Maximum 5 retries, total delay cannot
    exceed 60 seconds.
    """

    webhook_url: str
    """
    Use this field to override the URL for which Telnyx will send subsequent
    webhooks to for this call.
    """

    webhook_url_method: Literal["POST", "GET"]
    """HTTP request type used for `webhook_url`."""

    webhook_urls: Dict[str, str]
    """A map of event types to webhook URLs.

    When an event of the specified type occurs, the webhook URL associated with that
    event type will be called instead of `webhook_url`. Events not mapped here will
    use the default `webhook_url`.
    """

    webhook_urls_method: Literal["POST", "GET"]
    """HTTP request method to invoke `webhook_urls`."""


class WebhookRetriesPolicies(TypedDict, total=False):
    retries_ms: Iterable[int]
    """Array of delays in milliseconds between retry attempts.

    Total sum cannot exceed 60000ms.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from ..._models import BaseModel

__all__ = ["PortingEventSplitEvent", "Payload", "PayloadFrom", "PayloadPortingPhoneNumber", "PayloadTo"]


class PayloadFrom(BaseModel):
    """The porting order that was split."""

    id: Optional[str] = None
    """Identifies the porting order that was split."""


class PayloadPortingPhoneNumber(BaseModel):
    id: Optional[str] = None
    """Identifies the porting phone number that was moved."""


class PayloadTo(BaseModel):
    """The new porting order that the phone numbers was moved to."""

    id: Optional[str] = None
    """Identifies the porting order that was split."""


class Payload(BaseModel):
    """The webhook payload for the porting_order.split event"""

    from_: Optional[PayloadFrom] = FieldInfo(alias="from", default=None)
    """The porting order that was split."""

    porting_phone_numbers: Optional[List[PayloadPortingPhoneNumber]] = None
    """The list of porting phone numbers that were moved to the new porting order."""

    to: Optional[PayloadTo] = None
    """The new porting order that the phone numbers was moved to."""


class PortingEventSplitEvent(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies the event."""

    available_notification_methods: Optional[List[Literal["email", "webhook", "webhook_v1"]]] = None
    """Indicates the notification methods used."""

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    event_type: Optional[Literal["porting_order.split"]] = None
    """Identifies the event type"""

    payload: Optional[Payload] = None
    """The webhook payload for the porting_order.split event"""

    payload_status: Optional[Literal["created", "completed"]] = None
    """The status of the payload generation."""

    porting_order_id: Optional[str] = None
    """Identifies the porting order associated with the event."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal, TypeAlias

__all__ = ["EncryptedMedia"]

EncryptedMedia: TypeAlias = Optional[Literal["SRTP"]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["PortingAssociatedPhoneNumber", "PhoneNumberRange"]


class PhoneNumberRange(BaseModel):
    """Specifies the phone number range for this associated phone number."""

    end_at: Optional[str] = None
    """Specifies the end of the phone number range for this associated phone number."""

    start_at: Optional[str] = None
    """Specifies the start of the phone number range for this associated phone number."""


class PortingAssociatedPhoneNumber(BaseModel):
    id: Optional[str] = None
    """Uniquely identifies this associated phone number."""

    action: Optional[Literal["keep", "disconnect"]] = None
    """Specifies the action to take with this phone number during partial porting."""

    country_code: Optional[str] = None
    """Specifies the country code for this associated phone number.

    It is a two-letter ISO 3166-1 alpha-2 country code.
    """

    created_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was created."""

    phone_number_range: Optional[PhoneNumberRange] = None
    """Specifies the phone number range for this associated phone number."""

    phone_number_type: Optional[Literal["landline", "local", "mobile", "national", "shared_cost", "toll_free"]] = None
    """Specifies the phone number type for this associated phone number."""

    porting_order_id: Optional[str] = None
    """Identifies the porting order associated with this phone number."""

    record_type: Optional[str] = None
    """Identifies the type of the resource."""

    updated_at: Optional[datetime] = None
    """ISO 8601 formatted date indicating when the resource was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["PortingOrderRetrieveParams"]


class PortingOrderRetrieveParams(TypedDict, total=False):
    include_phone_numbers: bool
    """Include the first 50 phone number objects in the results"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AudioVisualizerConfig"]


class AudioVisualizerConfig(BaseModel):
    color: Optional[Literal["verdant", "twilight", "bloom", "mystic", "flare", "glacier"]] = None
    """The color theme for the audio visualizer."""

    preset: Optional[str] = None
    """The preset style for the audio visualizer."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from ..._models import BaseModel

__all__ = ["Conversation"]


class Conversation(BaseModel):
    id: str

    created_at: datetime
    """The datetime the conversation was created."""

    last_message_at: datetime
    """The datetime of the latest message in the conversation."""

    metadata: Dict[str, str]
    """Metadata associated with the conversation.

    Telnyx provides several pieces of metadata, but customers can also add their
    own. The reserved field `ai_disabled` (boolean) can be set to `true` to prevent
    AI-generated responses on this conversation. When `ai_disabled` is `true`, calls
    to the chat endpoint will return a 400 error. Set to `false` or remove the field
    to re-enable AI responses. This is useful when a human agent needs to take over
    the conversation mid-stream (e.g., a technician stepping in while AI was
    messaging a resident).
    """

    name: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict

import httpx

from ..types import global_ip_list_params, global_ip_create_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from .._base_client import AsyncPaginator, make_request_options
from ..types.global_ip_list_response import GlobalIPListResponse
from ..types.global_ip_create_response import GlobalIPCreateResponse
from ..types.global_ip_delete_response import GlobalIPDeleteResponse
from ..types.global_ip_retrieve_response import GlobalIPRetrieveResponse

__all__ = ["GlobalIPsResource", "AsyncGlobalIPsResource"]


class GlobalIPsResource(SyncAPIResource):
    """Global IPs"""

    @cached_property
    def with_raw_response(self) -> GlobalIPsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return GlobalIPsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> GlobalIPsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return GlobalIPsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        description: str | Omit = omit,
        name: str | Omit = omit,
        ports: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPCreateResponse:
        """
        Create a Global IP.

        Args:
          description: A user specified description for the address.

          name: A user specified name for the address.

          ports: A Global IP ports grouped by protocol code.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/global_ips",
            body=maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "ports": ports,
                },
                global_ip_create_params.GlobalIPCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPCreateResponse,
        )

    def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPRetrieveResponse:
        """
        Retrieve a Global IP.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get(
            f"/global_ips/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPRetrieveResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[GlobalIPListResponse]:
        """
        List all Global IPs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/global_ips",
            page=SyncDefaultFlatPagination[GlobalIPListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    global_ip_list_params.GlobalIPListParams,
                ),
            ),
            model=GlobalIPListResponse,
        )

    def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPDeleteResponse:
        """
        Delete a Global IP.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._delete(
            f"/global_ips/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPDeleteResponse,
        )


class AsyncGlobalIPsResource(AsyncAPIResource):
    """Global IPs"""

    @cached_property
    def with_raw_response(self) -> AsyncGlobalIPsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncGlobalIPsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncGlobalIPsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncGlobalIPsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        description: str | Omit = omit,
        name: str | Omit = omit,
        ports: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPCreateResponse:
        """
        Create a Global IP.

        Args:
          description: A user specified description for the address.

          name: A user specified name for the address.

          ports: A Global IP ports grouped by protocol code.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/global_ips",
            body=await async_maybe_transform(
                {
                    "description": description,
                    "name": name,
                    "ports": ports,
                },
                global_ip_create_params.GlobalIPCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPCreateResponse,
        )

    async def retrieve(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPRetrieveResponse:
        """
        Retrieve a Global IP.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._get(
            f"/global_ips/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPRetrieveResponse,
        )

    def list(
        self,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[GlobalIPListResponse, AsyncDefaultFlatPagination[GlobalIPListResponse]]:
        """
        List all Global IPs.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get_api_list(
            "/global_ips",
            page=AsyncDefaultFlatPagination[GlobalIPListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    global_ip_list_params.GlobalIPListParams,
                ),
            ),
            model=GlobalIPListResponse,
        )

    async def delete(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> GlobalIPDeleteResponse:
        """
        Delete a Global IP.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._delete(
            f"/global_ips/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=GlobalIPDeleteResponse,
        )


class GlobalIPsResourceWithRawResponse:
    def __init__(self, global_ips: GlobalIPsResource) -> None:
        self._global_ips = global_ips

        self.create = to_raw_response_wrapper(
            global_ips.create,
        )
        self.retrieve = to_raw_response_wrapper(
            global_ips.retrieve,
        )
        self.list = to_raw_response_wrapper(
            global_ips.list,
        )
        self.delete = to_raw_response_wrapper(
            global_ips.delete,
        )


class AsyncGlobalIPsResourceWithRawResponse:
    def __init__(self, global_ips: AsyncGlobalIPsResource) -> None:
        self._global_ips = global_ips

        self.create = async_to_raw_response_wrapper(
            global_ips.create,
        )
        self.retrieve = async_to_raw_response_wrapper(
            global_ips.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            global_ips.list,
        )
        self.delete = async_to_raw_response_wrapper(
            global_ips.delete,
        )


class GlobalIPsResourceWithStreamingResponse:
    def __init__(self, global_ips: GlobalIPsResource) -> None:
        self._global_ips = global_ips

        self.create = to_streamed_response_wrapper(
            global_ips.create,
        )
        self.retrieve = to_streamed_response_wrapper(
            global_ips.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            global_ips.list,
        )
        self.delete = to_streamed_response_wrapper(
            global_ips.delete,
        )


class AsyncGlobalIPsResourceWithStreamingResponse:
    def __init__(self, global_ips: AsyncGlobalIPsResource) -> None:
        self._global_ips = global_ips

        self.create = async_to_streamed_response_wrapper(
            global_ips.create,
        )
        self.retrieve = async_to_streamed_response_wrapper(
            global_ips.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            global_ips.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            global_ips.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from telnyx import Telnyx, AsyncTelnyx
from tests.utils import assert_matches_type
from telnyx.types import (
    SubNumberOrdersReportCreateResponse,
    SubNumberOrdersReportRetrieveResponse,
)
from telnyx._utils import parse_datetime

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestSubNumberOrdersReport:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create(self, client: Telnyx) -> None:
        sub_number_orders_report = client.sub_number_orders_report.create()
        assert_matches_type(SubNumberOrdersReportCreateResponse, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Telnyx) -> None:
        sub_number_orders_report = client.sub_number_orders_report.create(
            country_code="US",
            created_at_gt=parse_datetime("2023-04-05T10:22:08.230549Z"),
            created_at_lt=parse_datetime("2025-06-05T10:22:08.230549Z"),
            customer_reference="STRING",
            order_request_id="12ade33a-21c0-473b-b055-b3c836e1c293",
            status="success",
        )
        assert_matches_type(SubNumberOrdersReportCreateResponse, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Telnyx) -> None:
        response = client.sub_number_orders_report.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_orders_report = response.parse()
        assert_matches_type(SubNumberOrdersReportCreateResponse, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Telnyx) -> None:
        with client.sub_number_orders_report.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_orders_report = response.parse()
            assert_matches_type(SubNumberOrdersReportCreateResponse, sub_number_orders_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Telnyx) -> None:
        sub_number_orders_report = client.sub_number_orders_report.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c293",
        )
        assert_matches_type(SubNumberOrdersReportRetrieveResponse, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Telnyx) -> None:
        response = client.sub_number_orders_report.with_raw_response.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c293",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_orders_report = response.parse()
        assert_matches_type(SubNumberOrdersReportRetrieveResponse, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Telnyx) -> None:
        with client.sub_number_orders_report.with_streaming_response.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c293",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_orders_report = response.parse()
            assert_matches_type(SubNumberOrdersReportRetrieveResponse, sub_number_orders_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `report_id` but received ''"):
            client.sub_number_orders_report.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_method_download(self, client: Telnyx) -> None:
        sub_number_orders_report = client.sub_number_orders_report.download(
            "12ade33a-21c0-473b-b055-b3c836e1c293",
        )
        assert_matches_type(str, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_raw_response_download(self, client: Telnyx) -> None:
        response = client.sub_number_orders_report.with_raw_response.download(
            "12ade33a-21c0-473b-b055-b3c836e1c293",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_orders_report = response.parse()
        assert_matches_type(str, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_streaming_response_download(self, client: Telnyx) -> None:
        with client.sub_number_orders_report.with_streaming_response.download(
            "12ade33a-21c0-473b-b055-b3c836e1c293",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_orders_report = response.parse()
            assert_matches_type(str, sub_number_orders_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    def test_path_params_download(self, client: Telnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `report_id` but received ''"):
            client.sub_number_orders_report.with_raw_response.download(
                "",
            )


class TestAsyncSubNumberOrdersReport:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncTelnyx) -> None:
        sub_number_orders_report = await async_client.sub_number_orders_report.create()
        assert_matches_type(SubNumberOrdersReportCreateResponse, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncTelnyx) -> None:
        sub_number_orders_report = await async_client.sub_number_orders_report.create(
            country_code="US",
            created_at_gt=parse_datetime("2023-04-05T10:22:08.230549Z"),
            created_at_lt=parse_datetime("2025-06-05T10:22:08.230549Z"),
            customer_reference="STRING",
            order_request_id="12ade33a-21c0-473b-b055-b3c836e1c293",
            status="success",
        )
        assert_matches_type(SubNumberOrdersReportCreateResponse, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sub_number_orders_report.with_raw_response.create()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_orders_report = await response.parse()
        assert_matches_type(SubNumberOrdersReportCreateResponse, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sub_number_orders_report.with_streaming_response.create() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_orders_report = await response.parse()
            assert_matches_type(SubNumberOrdersReportCreateResponse, sub_number_orders_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncTelnyx) -> None:
        sub_number_orders_report = await async_client.sub_number_orders_report.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c293",
        )
        assert_matches_type(SubNumberOrdersReportRetrieveResponse, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sub_number_orders_report.with_raw_response.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c293",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_orders_report = await response.parse()
        assert_matches_type(SubNumberOrdersReportRetrieveResponse, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sub_number_orders_report.with_streaming_response.retrieve(
            "12ade33a-21c0-473b-b055-b3c836e1c293",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_orders_report = await response.parse()
            assert_matches_type(SubNumberOrdersReportRetrieveResponse, sub_number_orders_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `report_id` but received ''"):
            await async_client.sub_number_orders_report.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_method_download(self, async_client: AsyncTelnyx) -> None:
        sub_number_orders_report = await async_client.sub_number_orders_report.download(
            "12ade33a-21c0-473b-b055-b3c836e1c293",
        )
        assert_matches_type(str, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_raw_response_download(self, async_client: AsyncTelnyx) -> None:
        response = await async_client.sub_number_orders_report.with_raw_response.download(
            "12ade33a-21c0-473b-b055-b3c836e1c293",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        sub_number_orders_report = await response.parse()
        assert_matches_type(str, sub_number_orders_report, path=["response"])

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_streaming_response_download(self, async_client: AsyncTelnyx) -> None:
        async with async_client.sub_number_orders_report.with_streaming_response.download(
            "12ade33a-21c0-473b-b055-b3c836e1c293",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            sub_number_orders_report = await response.parse()
            assert_matches_type(str, sub_number_orders_report, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Mock server tests are disabled")
    @parametrize
    async def test_path_params_download(self, async_client: AsyncTelnyx) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `report_id` but received ''"):
            await async_client.sub_number_orders_report.with_raw_response.download(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["OAuthClientUpdateParams"]


class OAuthClientUpdateParams(TypedDict, total=False):
    allowed_grant_types: List[Literal["client_credentials", "authorization_code", "refresh_token"]]
    """List of allowed OAuth grant types"""

    allowed_scopes: SequenceNotStr[str]
    """List of allowed OAuth scopes"""

    logo_uri: str
    """URL of the client logo"""

    name: str
    """The name of the OAuth client"""

    policy_uri: str
    """URL of the client's privacy policy"""

    redirect_uris: SequenceNotStr[str]
    """List of redirect URIs"""

    require_pkce: bool
    """Whether PKCE (Proof Key for Code Exchange) is required for this client"""

    tos_uri: str
    """URL of the client's terms of service"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel
from .whatsapp_calling_settings_data import WhatsappCallingSettingsData

__all__ = ["CallingSettingRetrieveResponse"]


class CallingSettingRetrieveResponse(BaseModel):
    data: Optional[WhatsappCallingSettingsData] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel
from .call_recording_error import CallRecordingError

__all__ = ["CallRecordingErrorWebhookEvent"]


class CallRecordingErrorWebhookEvent(BaseModel):
    data: Optional[CallRecordingError] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import httpx

from ..._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncDefaultFlatPagination, AsyncDefaultFlatPagination
from ..._base_client import AsyncPaginator, make_request_options
from ...types.queues import call_list_params, call_update_params
from ...types.queues.call_list_response import CallListResponse
from ...types.queues.call_retrieve_response import CallRetrieveResponse

__all__ = ["CallsResource", "AsyncCallsResource"]


class CallsResource(SyncAPIResource):
    """Queue commands operations"""

    @cached_property
    def with_raw_response(self) -> CallsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return CallsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> CallsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return CallsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        call_control_id: str,
        *,
        queue_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallRetrieveResponse:
        """
        Retrieve an existing call from an existing queue

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        if not call_control_id:
            raise ValueError(f"Expected a non-empty value for `call_control_id` but received {call_control_id!r}")
        return self._get(
            f"/queues/{queue_name}/calls/{call_control_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallRetrieveResponse,
        )

    def update(
        self,
        call_control_id: str,
        *,
        queue_name: str,
        keep_after_hangup: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Update queued call's keep_after_hangup flag

        Args:
          keep_after_hangup: Whether the call should remain in queue after hangup.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        if not call_control_id:
            raise ValueError(f"Expected a non-empty value for `call_control_id` but received {call_control_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._patch(
            f"/queues/{queue_name}/calls/{call_control_id}",
            body=maybe_transform({"keep_after_hangup": keep_after_hangup}, call_update_params.CallUpdateParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        queue_name: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncDefaultFlatPagination[CallListResponse]:
        """
        Retrieve the list of calls in an existing queue

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        return self._get_api_list(
            f"/queues/{queue_name}/calls",
            page=SyncDefaultFlatPagination[CallListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    call_list_params.CallListParams,
                ),
            ),
            model=CallListResponse,
        )

    def remove(
        self,
        call_control_id: str,
        *,
        queue_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Removes an inactive call from a queue.

        If the call is no longer active, use this
        command to remove it from the queue.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        if not call_control_id:
            raise ValueError(f"Expected a non-empty value for `call_control_id` but received {call_control_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/queues/{queue_name}/calls/{call_control_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncCallsResource(AsyncAPIResource):
    """Queue commands operations"""

    @cached_property
    def with_raw_response(self) -> AsyncCallsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#accessing-raw-response-data-eg-headers
        """
        return AsyncCallsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncCallsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/team-telnyx/telnyx-python#with_streaming_response
        """
        return AsyncCallsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        call_control_id: str,
        *,
        queue_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> CallRetrieveResponse:
        """
        Retrieve an existing call from an existing queue

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        if not call_control_id:
            raise ValueError(f"Expected a non-empty value for `call_control_id` but received {call_control_id!r}")
        return await self._get(
            f"/queues/{queue_name}/calls/{call_control_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=CallRetrieveResponse,
        )

    async def update(
        self,
        call_control_id: str,
        *,
        queue_name: str,
        keep_after_hangup: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Update queued call's keep_after_hangup flag

        Args:
          keep_after_hangup: Whether the call should remain in queue after hangup.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        if not call_control_id:
            raise ValueError(f"Expected a non-empty value for `call_control_id` but received {call_control_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._patch(
            f"/queues/{queue_name}/calls/{call_control_id}",
            body=await async_maybe_transform(
                {"keep_after_hangup": keep_after_hangup}, call_update_params.CallUpdateParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def list(
        self,
        queue_name: str,
        *,
        page_number: int | Omit = omit,
        page_size: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[CallListResponse, AsyncDefaultFlatPagination[CallListResponse]]:
        """
        Retrieve the list of calls in an existing queue

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        return self._get_api_list(
            f"/queues/{queue_name}/calls",
            page=AsyncDefaultFlatPagination[CallListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "page_number": page_number,
                        "page_size": page_size,
                    },
                    call_list_params.CallListParams,
                ),
            ),
            model=CallListResponse,
        )

    async def remove(
        self,
        call_control_id: str,
        *,
        queue_name: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """Removes an inactive call from a queue.

        If the call is no longer active, use this
        command to remove it from the queue.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not queue_name:
            raise ValueError(f"Expected a non-empty value for `queue_name` but received {queue_name!r}")
        if not call_control_id:
            raise ValueError(f"Expected a non-empty value for `call_control_id` but received {call_control_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/queues/{queue_name}/calls/{call_control_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class CallsResourceWithRawResponse:
    def __init__(self, calls: CallsResource) -> None:
        self._calls = calls

        self.retrieve = to_raw_response_wrapper(
            calls.retrieve,
        )
        self.update = to_raw_response_wrapper(
            calls.update,
        )
        self.list = to_raw_response_wrapper(
            calls.list,
        )
        self.remove = to_raw_response_wrapper(
            calls.remove,
        )


class AsyncCallsResourceWithRawResponse:
    def __init__(self, calls: AsyncCallsResource) -> None:
        self._calls = calls

        self.retrieve = async_to_raw_response_wrapper(
            calls.retrieve,
        )
        self.update = async_to_raw_response_wrapper(
            calls.update,
        )
        self.list = async_to_raw_response_wrapper(
            calls.list,
        )
        self.remove = async_to_raw_response_wrapper(
            calls.remove,
        )


class CallsResourceWithStreamingResponse:
    def __init__(self, calls: CallsResource) -> None:
        self._calls = calls

        self.retrieve = to_streamed_response_wrapper(
            calls.retrieve,
        )
        self.update = to_streamed_response_wrapper(
            calls.update,
        )
        self.list = to_streamed_response_wrapper(
            calls.list,
        )
        self.remove = to_streamed_response_wrapper(
            calls.remove,
        )


class AsyncCallsResourceWithStreamingResponse:
    def __init__(self, calls: AsyncCallsResource) -> None:
        self._calls = calls

        self.retrieve = async_to_streamed_response_wrapper(
            calls.retrieve,
        )
        self.update = async_to_streamed_response_wrapper(
            calls.update,
        )
        self.list = async_to_streamed_response_wrapper(
            calls.list,
        )
        self.remove = async_to_streamed_response_wrapper(
            calls.remove,
        )
